from ase.units import fs, GPa
from ase.build import bulk
import asap3
from asap3.md.nvtberendsen import NVTBerendsen
from asap3.md.nptberendsen import NPTBerendsen, Inhomogeneous_NPTBerendsen
from asap3.md import MDLogger
from asap3.md.velocitydistribution import (MaxwellBoltzmannDistribution,
                                             Stationary)
from asap3.testtools import ReportTest
import numpy as np


def test_berendsen(pressure, homog=True):
    """Test NVT or NPT Berendsen dynamics.

    The pressure should be in atomic units.
    """
    rng = np.random.RandomState(None)
    a = bulk('Cu', orthorhombic=True).repeat((5, 5, 4))
    # Introduce an inhomogeneity
    a[7].symbol = 'Au'
    a[8].symbol = 'Au'
    del a[101]
    del a[100]
    print(a)
    a.calc = asap3.EMT()
    # Set temperature to 10 K
    MaxwellBoltzmannDistribution(a, temperature_K=10, force_temp=True, rng=rng)
    Stationary(a)
    ReportTest("Initial temperature", a.get_temperature(), 10, 0.0001)
    # Berendsen dynamics should raise this to 300 K
    T = 300
    if pressure is None:
        md = NVTBerendsen(a, timestep=4 * fs, temperature_K=T,
                              taut=2000*fs,
                              logfile='-', loginterval=500)
    elif homog:
        md = NPTBerendsen(a, timestep=4 * fs, temperature_K=T,
                              taut=2000*fs,
                              pressure_au=pressure, taup=2000*fs,
                              compressibility_au=1 / (140 * GPa))
        # We want logging with stress included
        md.attach(MDLogger(md, a, '-', stress=True), interval=500)
    else:
        md = Inhomogeneous_NPTBerendsen(
            a, timestep=4 * fs, temperature_K=T, taut=2000*fs,
            pressure_au=pressure, taup=2000*fs,
            compressibility_au=1 / (140 * GPa)
            )
        # We want logging with stress included
        md.attach(MDLogger(md, a, '-', stress=True), interval=500)
    md.run(steps=3000)
    # Now gather the temperature over 10000 timesteps, collecting it
    # every 5 steps
    temp = []
    press = []
    for i in range(1000):
        md.run(steps=5)
        temp.append(a.get_temperature())
        p = -a.get_stress(include_ideal_gas=True)[:3].sum() / 3.0
        press.append(p)
    temp = np.array(temp)
    avgtemp = np.mean(temp)
    fluct = np.std(temp)
    avgpressure = np.mean(press)
    print("Temperature is {:.2f} K +/- {:.2f} K.".format(avgtemp, fluct))
    print("Pressure is {:.4f} GPa.".format(avgpressure / GPa))
    ReportTest("Temperarure ({})".format(md.__class__), avgtemp, T, 10.0)
    if pressure is not None:
        ReportTest("Pressure ({})".format(md.__class__), avgpressure, pressure, 0.05 * GPa)

test_berendsen(None)

test_berendsen(1.0 * GPa)

test_berendsen(1.0 * GPa, False)

ReportTest.Summary()



from asap3 import *
from ase.cluster.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
from asap3.md.velocitydistribution import *
from ase.md.velocitydistribution import MaxwellBoltzmannDistribution as ASE_MaxwellBoltzmannDistribution
from ase.parallel import world

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]


# We need an asymmetric nanoparticle
surfaces = [(1, 0, 0), (1, 1, 1), (0, -1, 0), (-1, 0, 0)]
layers = [9, 7, 7, 8]
lc = 4.08000


###  Test the MaxwellBoltzmann distribution in parallel
if ismaster:
    atoms = FaceCenteredCubic('Au', surfaces, layers, latticeconstant=lc)
    atoms.center(vacuum=3.0)
else:
    atoms = None

atoms = MakeParallelAtoms(atoms, cpulayout)
MaxwellBoltzmannDistribution(atoms, temperature_K=300, force_temp=True)
ReportTest("Temperature of Maxwell-Boltzmann distribution", atoms.get_temperature(), 300, 1e-6)


### Test the following:
###    ParallelListOfAtoms.get_center_of_mass()
###    ParallelListOfAtoms.get_angular_momentum()
###    ParallelListOfAtoms.get_moments_of_inertial()
###    Stationary()
###    ZeroRotation()

# Create an atoms object with a nonzero angular momentum and center of mass momentum

if ismaster:
    atoms = FaceCenteredCubic('Au', surfaces, layers, latticeconstant=lc)
    atoms.center(vacuum=3.0)
    ASE_MaxwellBoltzmannDistribution(atoms, temperature_K=300, force_temp=True, communicator='serial')
    ReportTest("Temperature of serial Maxwell-Boltzmann distribution", atoms.get_temperature(), 300, 1e-6)
    atoms[0].momentum += np.array((0, 1.0, 0))
    init_com = atoms.get_center_of_mass()
    init_am = atoms.get_angular_momentum()
    init_mi = atoms.get_moments_of_inertia()
else:
    atoms = None

atoms = MakeParallelAtoms(atoms, cpulayout)
temp = atoms.get_temperature()
com = atoms.get_center_of_mass()
am = atoms.get_angular_momentum()
mi = atoms.get_moments_of_inertia()
if ismaster:
    for i, ax in enumerate('xyz'):
        ReportTest("Center of mass ({})".format(ax), com[i], init_com[i], 1e-6)
        ReportTest("Angular momentum ({})".format(ax), am[i], init_am[i], 1e-6)
        ReportTest("Moments of inertial ({})".format(ax), mi[i], init_mi[i], 1e-6)
    print("Angular momentum:", am)
    
Stationary(atoms)
ReportTest("Temperature after Stationary(atoms)", atoms.get_temperature(), temp, 1e-6)
ZeroRotation(atoms)
ReportTest("Temperature after ZeroRotation(atoms)", atoms.get_temperature(), temp, 1e-6)
am = atoms.get_angular_momentum()
for i, ax in enumerate('xyz'):
    ReportTest("Angular momentum ({}) is zero".format(ax), am[i], 0, 1e-6)


### Finalize

ReportTest.Summary()




from asap3 import *
from asap3.md.verlet import VelocityVerlet
from ase.lattice.cubic import FaceCenteredCubic
from asap3.io.trajectory import *
from ase.parallel import world
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution, Stationary
from asap3.constraints import FixAtoms
from asap3.testtools import ReportTest
from ase.io import read
import os

print_version(1)
#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]
else:
    cpulayout = 'auto'

delete = True
temp = 3000  # K

if ismaster:
    initial = FaceCenteredCubic(size=(10,10,10), symbol="Cu", pbc=(1,0,0))
else:
    initial = None
if isparallel:
    atoms = MakeParallelAtoms(initial, cpulayout)
    atoms.set_constraint(FixAtoms(mask=(atoms.get_ids() < atoms.get_global_number_of_atoms()/2)))
else:
    atoms = initial.copy()
    atoms.set_constraint(FixAtoms(indices=range(len(atoms)//2)))

atoms.set_calculator(EMT())
MaxwellBoltzmannDistribution(atoms, temperature_K=temp)
Stationary(atoms)

dyn = Langevin(atoms, 3*units.fs, temperature_K=temp, friction=0.02)
dyn.run(1000)

temps = [atoms.get_temperature()]
for i in range(10):
    dyn.run(10)
    temps.append(atoms.get_temperature())
    
avgtemp = np.mean(temps)
print("Average temperature T={:.1f}K  (task {})".format(avgtemp, world.rank))
c = atoms.constraints[0]
print("Removed DOF: {}   (task {})".format(c.get_removed_dof(atoms), world.rank))
ReportTest("Mean temperature:", avgtemp, temp, 200)

ReportTest.Summary()



"""Test the FixAtoms constraint and the Subset filter."""
from __future__ import print_function

from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution
from asap3.md.verlet import VelocityVerlet
from asap3.md.nvtberendsen import NVTBerendsen
from asap3.constraints import Filter, FixAtoms
from ase.constraints import FixAtoms as ASE_FixAtoms
from asap3.testtools import ReportTest
from asap3.mpi import world
import numpy as np

debug = 0
if debug == 1:
    DebugOutput("parallelconstraints%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

print_version(1)
#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

def sanity(name, atoms):
    initial = atoms.arrays['r_init']
    final = atoms.get_positions()
    fixed = atoms.get_tags().astype(bool)
    
    print(f"Sanity check, {name}:")
    ok = (final == initial) + np.logical_not(fixed)[:,np.newaxis]
    if ok.all():
        print("  Stationary atoms have not moved: OK")
    else:
        raise RuntimeError(f"Stationary atoms have moved ({name})")
    ok = (final != initial) + fixed[:,np.newaxis]
    if ok.all():
        print("  Mobile atoms have moved: OK")
    else:
        raise RuntimeError(f"Mobile atoms have not moved ({name})")

for dynamics in ("Verlet", "NVTBerendsen", "Langevin",):
    
    if ismaster:
        size = 5 * np.array(cpulayout)
        init = FaceCenteredCubic(size=size, symbol='Cu', pbc=False)
        z = init.get_positions()[:,2]
        fixedatoms = np.less(z, 0.501*z.max())
        print(len(init), sum(fixedatoms))
        MaxwellBoltzmannDistribution(init, temperature_K=6000)
        init.set_tags(fixedatoms)
    else:
        init = None


    print()
    print("Running simulation with Asap's FixAtoms")
    atoms2 = MakeParallelAtoms(init, cpulayout)
    atoms2.arrays['r_init'] = atoms2.get_positions()
    atoms2.set_calculator(EMT())
    atoms2.set_constraint(FixAtoms(mask=atoms2.get_tags().astype(bool)))

    if dynamics == "Verlet":
        dyn = VelocityVerlet(atoms2, 3*units.fs)
    elif dynamics == "NVTBerendsen":
        dyn = NVTBerendsen(atoms2, 3*units.fs, temperature_K=2000, taut=200*units.fs)
    elif dynamics == "Langevin":
        dyn = Langevin(atoms2, 3*units.fs, temperature_K=2000, friction=1e-3)
    else:
        assert False
    dyn.run(1000)

    sanity(dynamics+"+FixAtoms", atoms2)

    x = atoms2.get_positions()[:,0]
    fixedatoms2 = np.less(x, 0.501 * x.max())
    atoms2.set_tags(fixedatoms2)
    atoms2.arrays['r_init'] = atoms2.get_positions()
    atoms2.set_constraint(FixAtoms(mask=fixedatoms2))

    dyn.run(1000)

    sanity(dynamics+"+new-FixAtoms", atoms2)

print("ALL TESTS SUCCEEDED!")






from asap3 import *
from asap3.md.verlet import VelocityVerlet
from ase.lattice.cubic import FaceCenteredCubic
from asap3.io.trajectory import *
from ase.parallel import world
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution, Stationary
from asap3.constraints import FixAtoms
from ase.io import read
import os

print_version(1)
#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]
else:
    cpulayout = 'auto'

delete = True
filename = "fixatoms.traj"

if ismaster:
    initial = FaceCenteredCubic(size=(10,10,10), symbol="Cu", pbc=(1,0,0))
else:
    initial = None
if isparallel:
    atoms = MakeParallelAtoms(initial, cpulayout)
    atoms.set_constraint(FixAtoms(mask=(atoms.get_ids() < 5)))
else:
    atoms = initial.copy()
    atoms.set_constraint(FixAtoms(indices=range(5)))

atoms.set_calculator(EMT())
MaxwellBoltzmannDistribution(atoms, temperature_K=5000)
Stationary(atoms)

dyn = VelocityVerlet(atoms, 3*units.fs)
traj = Trajectory(filename, "w", atoms)
dyn.attach(traj, interval=10)
dyn.run(50)

e = atoms.get_potential_energy()

atoms2 = read(filename)
e2 = atoms2.get_potential_energy()

world.barrier()
if delete and world.rank == 0:
    os.remove(filename)
    
world.barrier()



from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
from asap3.md.velocitydistribution import *
from asap3.analysis import CoordinationNumbers, PTM
from ase.parallel import world
from ase.visualize import view

debug = 1
if debug == 1:
    DebugOutput("safe_nblist%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

# Important: always at least two cores along x direction
ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [3,1,1]
elif world.size == 4:
    cpulayout = [2,1,2]


def make_atoms():
    """Make an atomic system.

    Some vacuum is added, so that shearing it changes the number of
    atoms on the different MPI tasks.
    """
    if ismaster:
        atoms = FaceCenteredCubic('Cu', size=(20,20,20))
        atoms.pbc = False
        atoms.center(vacuum=5.0, axis=0)
        atoms.pbc = True
    else:
        atoms = None
    if world.size > 1:
        atoms = MakeParallelAtoms(atoms, cpulayout)
    atoms.calc = EMT()
    atoms.get_forces()
    return atoms

def deform_atoms(atoms):
    """Shear the atoms."""
    r = atoms.get_positions()
    df =  0.1 * r[:,1]
    r[:,0] += df
    atoms.set_positions(r)
    
atoms = make_atoms()
e0 = atoms.get_potential_energy()
deform_atoms(atoms)
e1 = atoms.get_potential_energy()
if world.rank == 0:
    print("\n\nEnergy of atoms changed from {} to {}".format(e0, e1), file=sys.stderr)

atoms = make_atoms()
print("A: {} atoms on cpu {}".format(len(atoms), world.rank), file=sys.stderr)
deform_atoms(atoms)
#atoms.get_potential_energy()    # Workaround
CoordinationNumbers(atoms)
atoms.get_forces()
e2 = atoms.get_potential_energy()
print("B: {} atoms on cpu {}".format(len(atoms), world.rank), file=sys.stderr)
ReportTest("Energy after CoordinationNumbers()", e2, e1, 0.001)


atoms = make_atoms()
deform_atoms(atoms)
#atoms.get_potential_energy()    # Workaround
nbl = atoms.calc.get_neighborlist()
nbl.check_and_update(atoms)
atoms.get_forces()
e2 = atoms.get_potential_energy()
ReportTest("Energy after borrowed neighborlist update", e2, e1, 0.001)

atoms = make_atoms()
deform_atoms(atoms)
#atoms.get_potential_energy()    # Workaround
PTM(atoms, cutoff=4.7)
atoms.get_forces()
e2 = atoms.get_potential_energy()
ReportTest("Energy after PTM()", e2, e1, 0.001)


# The following are not expected to be affected by issue #43.

for i, nbtype in enumerate((FullNeighborList, NeighborCellLocator)):
    atoms = make_atoms()
    nbl = nbtype(4.8, atoms)
    deform_atoms(atoms)
    #atoms.get_potential_energy()    # Workaround
    nbl.check_and_update(atoms)
    atoms.get_forces()
    e2 = atoms.get_potential_energy()
    ReportTest("Energy neighbor list type {}".format(i), e2, e1, 0.001)

ReportTest.Summary()



"""Test the FixAtoms constraint and the Subset filter."""

from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution
from asap3.md.verlet import VelocityVerlet
from asap3.md.nvtberendsen import NVTBerendsen
from ase.constraints import Filter
from asap3.constraints import FixAtoms
from ase.constraints import FixAtoms as ASE_FixAtoms
from asap3.testtools import ReportTest
import numpy as np

def sanity(name, initial, final, fixed):
    print(f"Sanity check, {name}:")
    ok = (final == initial) + np.logical_not(fixed)[:,np.newaxis]
    if ok.all():
        print("  Stationary atoms have not moved: OK")
    else:
        raise RuntimeError(f"Stationary atoms have moved ({name})")
    ok = (final != initial) + fixed[:,np.newaxis]
    if ok.all():
        print("  Mobile atoms have moved: OK")
    else:
        raise RuntimeError(f"Mobile atoms have not moved ({name})")

for dynamics in ("Verlet", "NVTBerendsen", "Langevin"):
    init = FaceCenteredCubic(size=(10,10,10), symbol='Cu', pbc=False)
    z = init.get_positions()[:,2]
    fixedatoms = np.less(z, 0.501*z.max())
    print(len(init), sum(fixedatoms))
    MaxwellBoltzmannDistribution(init, temperature_K=2000)
    r_init = init.get_positions()

    print()
    print("Running simulation with Asap's FixAtoms")
    atoms2 = Atoms(init)
    atoms2.set_calculator(EMT())
    atoms2.set_constraint(FixAtoms(mask=fixedatoms))

    if dynamics == "Verlet":
        dyn = VelocityVerlet(atoms2, 3*units.fs)
    elif dynamics == "NVTBerendsen":
        dyn = NVTBerendsen(atoms2, 3*units.fs, temperature_K=2000, taut=200*units.fs)
    elif dynamics == "Langevin":
        dyn = Langevin(atoms2, 3*units.fs, temperature_K=2000, friction=1e-3)
    else:
        assert False
    dyn.run(50)
    r2 = atoms2.get_positions()

    sanity(dynamics+"+FixAtoms", r_init, r2, fixedatoms)

    x = r2[:,0]
    fixedatoms2 = np.less(x, 0.501 * x.max())
    print(len(atoms2), sum(fixedatoms2))

    print("Running simulation with new FixAtoms")
    atoms2.set_constraint(FixAtoms(mask=fixedatoms2))
    #dyn = VelocityVerlet(atoms2, 2*units.fs)

    dyn.run(50)
    r3 = atoms2.get_positions()

    sanity(dynamics+"+new-FixAtoms", r2, r3, fixedatoms2)



print("ALL TESTS SUCCEEDED")






from ase.units import fs, kB
from ase.build import bulk
import asap3
from asap3.md.langevin import Langevin
from asap3.md.velocitydistribution import (MaxwellBoltzmannDistribution,
                                             Stationary)
from asap3.testtools import ReportTest

import numpy as np

# rng = np.random.RandomState(0)
a = bulk('Au', cubic=True).repeat((5, 5, 5))
a.pbc = (False, False, False)
a.center(vacuum=2.0)
print(a)
a.calc = asap3.EMT()
# Set temperature to 10 K
MaxwellBoltzmannDistribution(a, temperature_K=10, force_temp=True)
Stationary(a)
ReportTest("Initial temperature", a.get_temperature(), 10, 0.0001)
# Langevin dynamics should raise this to 300 K
T = 300
md = Langevin(a, timestep=4 * fs, temperature_K=T, friction=0.01,
                  logfile='-', loginterval=500)
md.run(steps=5000)
# Now gather the temperature over 10000 timesteps, collecting it
# every 5 steps
temp = []
energy = []
com = a.get_center_of_mass()
for i in range(2000):
    md.run(steps=5)
    temp.append(a.get_temperature())
    energy.append(a.get_potential_energy() + a.get_kinetic_energy())
temp = np.array(temp)
avgtemp = np.mean(temp)
fluct = np.std(temp)
avgenergy = np.mean(energy)
print("Temperature is {:.2f} K +/- {:.2f} K".format(avgtemp, fluct))
ReportTest("Average temperature", avgtemp, T, 10.0)

delta = np.linalg.norm(a.get_center_of_mass() - com)
ReportTest("Change in center of mass", delta, 0.0, 1e-9)

ReportTest.Summary()





from asap3 import *
from asap3.testtools import ReportTest
from asap3.mpi import world
from asap3.io import Trajectory
from ase.io import write
from ase.build import bulk

print_version(1)

if world.rank == 0:
    atoms1 = bulk("Cu", cubic=True).repeat((20,20,20))
    atoms1.set_pbc = False
    atoms1.center(vacuum=5.)
    ReportTest("Number of atoms", len(atoms1), 32000, 0)
    atoms1.set_calculator(EMT())
    e0 = atoms1.get_potential_energy()
    write("serialatoms.traj", atoms1, parallel=False)
else:
    atoms1 = None

world.barrier()
# atoms = MakeParallelAtoms(atoms1, 'auto')
# atoms.set_calculator(EMT())
# e = atoms.get_potential_energy()
# ReportTest("Energy of distributed atoms", e, e0, 1e-3)

atoms = Trajectory("serialatoms.traj").get_atoms(-1, 'auto')
atoms.set_calculator(EMT())
e = atoms.get_potential_energy()
world.barrier()   # Protects cleanup
if world.rank == 0:
    os.unlink('serialatoms.traj')
    ReportTest("Energy of saved atoms", e, e0, 1e-3)

world.barrier()
    
ReportTest.Summary()



from asap3.md.verlet import VelocityVerlet
from asap3.md.langevin import Langevin
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution
from asap3 import EMT
from asap3.testtools import ReportTest
from ase.build import bulk
from ase import units
import numpy as np

# Global parameters:
T=300
timestep=5

def testmd(atoms, dynclass, name, dynkwargs={}):
    MaxwellBoltzmannDistribution(atoms, temperature_K=T*2, force_temp=True)
    atoms.set_calculator(EMT())
    dyn = dynclass(atoms, timestep=timestep*units.fs, **dynkwargs)
    tt =  atoms.get_temperature()
    print("Initial temperature:", tt)
    ReportTest("Initial temperature ({})".format(name), tt, 2*T, 1e-9)
    p = atoms.get_momenta()
    print("Initial square momenta:", (p*p).sum())
    dyn.run(500)
    tt =  atoms.get_temperature()
    print("Final temperature:", tt)
    ReportTest("Final temperature ({})".format(name), tt, T, 20.0)
    p = atoms.get_momenta()
    print("Final square momenta:", (p*p).sum())
    print()

mass_Cu = 63.546
size = 7

atoms = bulk('Cu', cubic=True).repeat((size,size,size))
assert len(atoms) == 4*size**3
testmd(atoms, VelocityVerlet, "Verlet - normal")

atoms = bulk('Cu', cubic=True).repeat((size,size,size))
atoms.set_masses(0.5*mass_Cu*np.ones(len(atoms)))
testmd(atoms, VelocityVerlet, "Verlet - low mass")

lgvkw = {'temperature_K': T, 'friction':0.05}
atoms = bulk('Cu', cubic=True).repeat((size,size,size))
testmd(atoms, Langevin, "Langevin - normal", lgvkw)

atoms = bulk('Cu', cubic=True).repeat((size,size,size))
atoms.set_masses(0.5*mass_Cu*np.ones(len(atoms)))
testmd(atoms, Langevin, "Langevin - low mass", lgvkw)

ReportTest.Summary()



from ase.units import fs, GPa
from ase.build import bulk
import asap3
from asap3.md.npt import NPT
from asap3.md.nptberendsen import NPTBerendsen
from asap3.md import MDLogger
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution, Stationary
from asap3.testtools import ReportTest
import numpy as np


def test_npt_asap():
    pressure = 1.0 * GPa
    rng = np.random.RandomState(0)
    a = bulk('Cu', orthorhombic=True).repeat((6, 6, 4))
    print(a)
    a.calc = asap3.EMT()
    # NPT Dynamics is awful at setting correct pressure and
    # temperature, but should be good at maintaining it.  Use
    # NPTBerendsen to hit the right value.
    T = 300
    MaxwellBoltzmannDistribution(a, temperature_K=T, force_temp=True, rng=rng)
    Stationary(a)
    berend = NPTBerendsen(a, timestep=4 * fs, temperature_K=T,
                          taut=2000*fs,
                          pressure_au=pressure, taup=2000*fs,
                          compressibility_au=1 / (140 * GPa),
                          logfile='-', loginterval=500)
    berend.run(steps=3000)
    # Now gather the temperature over 10000 timesteps, collecting it every 5 steps
    ptime = 2000 * fs
    md = NPT(a, timestep=4 * fs, temperature_K=T, externalstress=pressure,
                 ttime=2000 * fs, pfactor=ptime**2/(140 / GPa))
    # We want logging with stress included
    md.attach(MDLogger(md, a, '-', stress=True), interval=500)
    temp = []
    press = []
    for i in range(2000):
        md.run(steps=5)
        temp.append(a.get_temperature())
        p = -a.get_stress(include_ideal_gas=True)[:3].sum() / 3.0
        press.append(p)
    temp = np.array(temp)
    avgtemp = np.mean(temp)
    fluct = np.std(temp)
    avgpressure = np.mean(press)
    print("Temperature is {:.2f} K +/- {:.2f} K.".format(avgtemp, fluct))
    print("Pressure is {:.4f} GPa.".format(avgpressure / GPa))
    ReportTest("Temperature", avgtemp, T, 10.0)
    ReportTest("Pressure", avgpressure, pressure, 0.02 * GPa)


test_npt_asap()



from asap3 import *
from ase.cluster.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
from asap3.md.velocitydistribution import *
from asap3.analysis import CoordinationNumbers
from ase.md.velocitydistribution import MaxwellBoltzmannDistribution as ASE_MaxwellBoltzmannDistribution
from ase.parallel import world

debug = 0
if debug == 1:
    DebugOutput("deleteatoms%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]


# We need an asymmetric nanoparticle
surfaces = [(1, 0, 0), (1, 1, 1), (0, -1, 0), (-1, 0, 0)]
layers = [9, 7, 7, 8]
lc = 4.08000
if ismaster:
    atoms = FaceCenteredCubic('Au', surfaces, layers, latticeconstant=lc)
    atoms.center(vacuum=3.0)
else:
    atoms = None

atoms = MakeParallelAtoms(atoms, cpulayout)
MaxwellBoltzmannDistribution(atoms, temperature_K=600)
atoms.calc = EMT()

dyn = Langevin(atoms, 0.5*units.fs, temperature_K=600, friction=0.01)
dyn.run(1)

natoms = atoms.get_global_number_of_atoms()
zap = [0, natoms//2, natoms-1]
print("Atoms on {} (before):".format(world.rank), len(atoms))
atoms.delete_atoms_globally(global_indices=zap)
print("Atoms on {} (after):".format(world.rank), len(atoms))

ReportTest("Number of atoms reduced by 3", atoms.get_global_number_of_atoms(), natoms - len(zap), 0)
cn = CoordinationNumbers(atoms)
ReportTest("Number of coordination numbers", len(cn), len(atoms), 0)

dyn.run(15)
print("Simulation 1 completed without crashing.")

natoms = atoms.get_global_number_of_atoms()

# Try with different info on different tasks.
if world.size == 2:
    if world.rank == 0:
        zap = [0, natoms//2]
    else:
        zap = [natoms//2, natoms-1]
else:
    if world.rank == 1:
        zap = (natoms//2, natoms-1)
    elif world.rank == 2:
        zap = np.array([natoms//2, 0])
    else:
        zap = []
atoms.delete_atoms_globally(global_indices=zap)

dyn.run(15)

ReportTest("Number of atoms reduced by 3 (inhomogeneous case)", atoms.get_global_number_of_atoms(), natoms - 3, 0)
print("Simulation 2 completed without crashing.")







ReportTest.Summary()



"""Test the FixAtoms constraint and the Subset filter."""
from __future__ import print_function

from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution
from asap3.md.verlet import VelocityVerlet
from asap3.md.nvtberendsen import NVTBerendsen
from asap3.constraints import Filter, FixAtoms
from ase.constraints import FixAtoms as ASE_FixAtoms
from asap3.testtools import ReportTest
from asap3.mpi import world
import numpy as np

debug = 0
if debug == 1:
    DebugOutput("parallelconstraints%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

print_version(1)
#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

if ismaster:
    size = 5 * np.array(cpulayout)
    init = FaceCenteredCubic(size=size, symbol='Cu', pbc=False)
    z = init.get_positions()[:,2]
    fixedatoms = np.less(z, 0.501*z.max())
    print(len(init), sum(fixedatoms))
    MaxwellBoltzmannDistribution(init, temperature_K=6000)
    init.set_tags(fixedatoms)
else:
    init = None

print()
print("Running simulation with Filter")
atoms1 = MakeParallelAtoms(init, cpulayout)
atoms1.arrays['r_init'] = atoms1.get_positions()
atoms1.set_calculator(EMT())
atoms1a = Filter(atoms1, mask=np.logical_not(atoms1.get_tags()))

dyn = VelocityVerlet(atoms1a, 3*units.fs)
dyn.run(1000)

print()
print("Running simulation with Asap's FixAtoms")
atoms2 = MakeParallelAtoms(init, cpulayout)
atoms2.arrays['r_init'] = atoms2.get_positions()
atoms2.set_calculator(EMT())
atoms2.set_constraint(FixAtoms(mask=atoms2.get_tags().astype(bool)))

dyn = VelocityVerlet(atoms2, 3*units.fs)
dyn.run(1000)

print()
print("Running NPTBerendsen simulation with Asap's FixAtoms")
atoms3 = MakeParallelAtoms(init, cpulayout)
atoms3.arrays['r_init'] = atoms3.get_positions()
atoms3.set_calculator(EMT())
atoms3.set_constraint(FixAtoms(mask=atoms3.get_tags().astype(bool)))

dyn = NVTBerendsen(atoms3, 3*units.fs, temperature_K=3000, taut=200*units.fs)
dyn.run(1000)


print()
print("Running Langevin simulation with Asap's FixAtoms")
atoms4 = MakeParallelAtoms(init, cpulayout)
atoms4.arrays['r_init'] = atoms4.get_positions()
atoms4.set_calculator(EMT())
atoms4.set_constraint(FixAtoms(mask=atoms4.get_tags().astype(bool)))

dyn = Langevin(atoms4, 3*units.fs, temperature_K=3000, friction=0.01)
dyn.run(1000)

print()
print("Running Verlet then Langevin simulation with Asap's FixAtoms")
atoms5 = MakeParallelAtoms(init, cpulayout)
atoms5.arrays['r_init'] = atoms5.get_positions()
atoms5.set_calculator(EMT())
atoms5.set_constraint(FixAtoms(mask=atoms5.get_tags().astype(bool)))

dyn = VelocityVerlet(atoms5, 3*units.fs)
dyn.run(1000)

dyn = Langevin(atoms5, 3*units.fs, temperature_K=3000, friction=0.01)
dyn.run(1000)


print()
print("Running NVTBerendsen then Verlet simulation with Asap's FixAtoms")
atoms6 = MakeParallelAtoms(init, cpulayout)
atoms6.arrays['r_init'] = atoms6.get_positions()
atoms6.set_calculator(EMT())
atoms6.set_constraint(FixAtoms(mask=atoms6.get_tags().astype(bool)))

dyn = NVTBerendsen(atoms6, 3*units.fs, temperature_K=3000, taut=200*units.fs)
dyn.run(1000)

dyn = VelocityVerlet(atoms6, 3*units.fs)
dyn.run(1000)


print()
print("Running Verlet then NVTBerendsen simulation with Asap's FixAtoms")
atoms7 = MakeParallelAtoms(init, cpulayout)
atoms7.arrays['r_init'] = atoms7.get_positions()
atoms7.set_calculator(EMT())
atoms7.set_constraint(FixAtoms(mask=atoms7.get_tags().astype(bool)))

dyn = VelocityVerlet(atoms7, 3*units.fs)
dyn.run(1000)

dyn = NVTBerendsen(atoms7, 3*units.fs, temperature_K=3000, taut=200*units.fs)
dyn.run(1000)

print()
sanity = [[atoms1, "Verlet + Filter"],
          [atoms2, "Verlet + Asap's FixAtoms"],
          [atoms3, "NVTBerendsen + Asap's FixAtoms"],
          [atoms4, "Langevin + Asap's FixAtoms"],
          [atoms5, "Verlet + Langevin + Asap's FixAtoms"],
          [atoms6, "NVTBerendsen + Verlet + Asap's FixAtoms"],
          [atoms7, "Verlet + NVTBerendsen + Asap's FixAtoms"],
          ]
for a, label in sanity:
    print(world.rank, "Sanity check, %s:" % (label,))
    r_init = a.arrays['r_init']
    ok = (a.get_positions() == r_init) + np.logical_not(a.get_tags())[:,np.newaxis]
    if ok.all():
        print(world.rank, "  Stationary atoms have not moved: OK")
    else:
        raise RuntimeError("Stationary atoms have moved (%s)" % (label,))
    ok = (a.get_positions() != r_init) + a.get_tags()[:,np.newaxis]
    if ok.all():
        print(world.rank, "  Mobile atoms have moved: OK")
    else:
        raise RuntimeError("Mobile atoms have not moved (%s)" % (label,))
    

print("ALL TESTS SUCCEEDED!")






from __future__ import print_function
import numpy as np

from ase import Atoms, units
from ase.data import atomic_numbers
from asap3 import Morse, PropertyNotImplementedError

# Copied from old version of ase.test.testsuite
class must_raise:
    """Context manager for checking raising of exceptions."""
    def __init__(self, exception):
        self.exception = exception

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_value, tb):
        if exc_type is None:
            raise RuntimeError('Failed to fail: ' + str(self.exception))
        return issubclass(exc_type, self.exception)

# Define constants and calculator
elements = np.array([atomic_numbers['Ru'], atomic_numbers['Ar']])
epsilon = np.array([[5.720, 0.092], [0.092, 0.008]])
alpha = np.array([[1.475, 2.719], [2.719, 1.472]])
rmin = np.array([[2.110, 2.563], [2.563, 4.185]])
rcut = rmin.max() + 6.0 / alpha.min()

calc = Morse(elements, epsilon, alpha, rmin)
atoms = Atoms(['Ar', 'Ar'], [[0.0, 0.0, 0.0], [rcut + 1.0, 0.0, 0.0]])
atoms.center(vacuum=10.0)
atoms.set_pbc(True)
atoms.set_calculator(calc)

# This should not fail
energy = atoms.get_potential_energy()

#This should fail
with must_raise(PropertyNotImplementedError):
    stress = atoms.get_stress()

